// Fill out your copyright notice in the Description page of Project Settings.


#include "AgentPlatformBlueprintLibrary.h"

FString UAgentPlatformBlueprintLibrary::GetKeyValue(FString values, FString key)
{
	TArray<FString> args;
	TArray<FString> parms;
	int32 tot = values.ParseIntoArrayWS(args);
	for (int32 i = 0; i < tot; i++) 
	{

		int loc = args[i].Find(key);
		if (loc >= 0) 
		{
			int32 tot2 = args[i].ParseIntoArray(parms, TEXT("="));
			if (tot2 == 2) 
			{
				return parms[1];
			}
		}
	}
	return FString();
}

float UAgentPlatformBlueprintLibrary::GetScale(float val1, float val2, float val3, float val4) {
	return (val2 - val1) / (val4 - val3);
}

FVector UAgentPlatformBlueprintLibrary::LatLongToXY(FVector coordinates, FVector2D XY_1, FVector2D XY_2, FVector2D LAT_LONG_1, FVector2D LAT_LONG_2)
{
	float Sx = UAgentPlatformBlueprintLibrary::GetScale(XY_1.X, XY_2.X, LAT_LONG_1.X, LAT_LONG_2.X);
	float Sy = UAgentPlatformBlueprintLibrary::GetScale(XY_1.Y, XY_2.Y, LAT_LONG_1.Y, LAT_LONG_2.Y);

	float x = XY_1.X + (coordinates.X - LAT_LONG_1.X) * Sx;
	float y = XY_1.Y + (coordinates.Y - LAT_LONG_1.Y) * Sy;

	return FVector(x, y, coordinates.Z);
}

FVector UAgentPlatformBlueprintLibrary::XYToLatLong(FVector coordinates, FVector2D XY_1, FVector2D XY_2, FVector2D LAT_LONG_1, FVector2D LAT_LONG_2)
{
	float Sx = UAgentPlatformBlueprintLibrary::GetScale(XY_1.X, XY_2.X, LAT_LONG_1.X, LAT_LONG_2.X);
	float Sy = UAgentPlatformBlueprintLibrary::GetScale(XY_1.Y, XY_2.Y, LAT_LONG_1.Y, LAT_LONG_2.Y);

	float x = LAT_LONG_1.X + (coordinates.X - XY_1.X) / Sx;
	float y = LAT_LONG_1.Y + (coordinates.Y - XY_1.Y) / Sy;

	
	return FVector(x, y, 0);
}

FString UAgentPlatformBlueprintLibrary::JsonGetValue(FString json, FString keyValue)
{
	TSharedRef<TJsonReader<>> reader = TJsonReaderFactory<>::Create(json);
	TSharedPtr<FJsonObject> jsonObject;
	FJsonSerializer::Deserialize(reader, jsonObject);
	TMap<FString, TSharedPtr<FJsonValue>> values = jsonObject.Get()->Values;
	FString retVal =  UAgentPlatformBlueprintLibrary::Find(values, keyValue);
	return retVal;

}


FAgentPlatformJsonObject UAgentPlatformBlueprintLibrary::JsonGetObject(FString json)
{
	FAgentPlatformJsonObject object;
	if (json.IsEmpty()) 
	{
		return object;
	}
	TSharedRef<TJsonReader<>> reader = TJsonReaderFactory<>::Create(json);
	TSharedPtr<FJsonObject> jsonObject;
	FJsonSerializer::Deserialize(reader, object.Object);

	return object;
}

FAgentPlatformJsonObject UAgentPlatformBlueprintLibrary::JsonAddValue(FString key, FString value, FAgentPlatformJsonObject jsonObject)
{
	FAgentPlatformJsonObject retObject;
	if (jsonObject.Object != nullptr) 
	{
		retObject.Object = jsonObject.Object;
	}
	else 
	{
		retObject.Object = MakeShared<FJsonObject>();
	}
	FJsonValueString jsonValue(value);
	retObject.Object.Get()->SetStringField(key, value);
	return retObject;
}

FAgentPlatformJsonObject UAgentPlatformBlueprintLibrary::JsonAddObject(FString key, FAgentPlatformJsonObject newObject, FAgentPlatformJsonObject jsonObject)
{
	FAgentPlatformJsonObject retObject;
	if (jsonObject.Object != nullptr) 
	{
		retObject.Object = jsonObject.Object;
	}
	else 
	{
		retObject.Object = MakeShared<FJsonObject>();
	}
	retObject.Object.Get()->SetObjectField(key, newObject.Object);
	return retObject;
}

FString UAgentPlatformBlueprintLibrary::JsonToString(FAgentPlatformJsonObject jsonObject)
{
	FString OutputString;
	TSharedRef<TJsonWriter<TCHAR, TCondensedJsonPrintPolicy<TCHAR>>> Writer = TJsonWriterFactory<TCHAR, TCondensedJsonPrintPolicy<TCHAR>>::Create(&OutputString, 0);
	FJsonSerializer::Serialize(jsonObject.Object.ToSharedRef(), Writer);
	return OutputString;
}

FString UAgentPlatformBlueprintLibrary::Find(TMap<FString, TSharedPtr<FJsonValue>> values, FString keyValue)
{
	TMap<FString, TSharedPtr<FJsonValue>> vals;
	for (auto& Elem : values) 
	{
		FString key = Elem.Key;
		FString valueStr = Elem.Value->AsString();
		if (valueStr.IsEmpty()) 
		{
			vals = Elem.Value->AsObject().Get()->Values;
			valueStr =  UAgentPlatformBlueprintLibrary::Find(vals, keyValue);
			if (!valueStr.IsEmpty()) 
			{
				return valueStr;
			}
		}
		if (key == keyValue) 
		{
			return valueStr;
		}
	}
	return FString();
}




