// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Kismet/BlueprintFunctionLibrary.h"
#include "Json/Public/Serialization/JsonReader.h"
#include "Json/Public/Serialization/JsonSerializer.h"
#include "Json/Public/Dom/JsonObject.h"
#include "JsonUtilities/Public/JsonObjectConverter.h"
#include "Core/Public/Containers/UnrealString.h"
#include "Policies/CondensedJsonPrintPolicy.h"
#include "AgentPlatformBlueprintLibrary.generated.h"


USTRUCT(BlueprintType)
struct FAgentPlatformJsonObject
{
	GENERATED_USTRUCT_BODY()
	FAgentPlatformJsonObject() : Object(NULL) {};
	TSharedPtr<class FJsonObject> Object;
};
USTRUCT(BlueprintType)
struct FAgentPlatformJsonValue
{
	GENERATED_USTRUCT_BODY()
	TSharedPtr<class FJsonValue> Value;
};


/**
 * 
 */
UCLASS()
class AGENTPLUGIN_API UAgentPlatformBlueprintLibrary : public UBlueprintFunctionLibrary
{
	GENERATED_BODY()
public:
	static float GetScale(float val1, float val2, float val3, float val4);
	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent")
	static FString GetKeyValue(FString values, FString key);
	

	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent World")
		static FVector LatLongToXY(FVector coordinates, FVector2D XY_1, FVector2D XY_2, FVector2D LAT_LONG_1, FVector2D LAT_LONG_2);

	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent World")
		static FVector XYToLatLong(FVector coordinates, FVector2D XY_1, FVector2D XY_2, FVector2D LAT_LONG_1, FVector2D LAT_LONG_2);

	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent")
		static FString JsonGetValue(FString json, FString keyValue);
	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent")
		static FAgentPlatformJsonObject JsonGetObject(FString json);
	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent")
		static FAgentPlatformJsonObject JsonAddValue(FString key, FString value, FAgentPlatformJsonObject jsonObject);
	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent")
		static FAgentPlatformJsonObject JsonAddObject(FString key, FAgentPlatformJsonObject newObject, FAgentPlatformJsonObject jsonObject);
	UFUNCTION(BlueprintCallable, Category = "Pureweb Agent")
		static FString JsonToString(FAgentPlatformJsonObject jsonObject);

	static FString Find(TMap<FString, TSharedPtr<FJsonValue>> values, FString keyValue);

};
